#include <windows.h>
#include <vector>
#include <math.h>
#include <fstream>

using namespace std;

HANDLE g_WriteHandle;
HANDLE g_ReadHandle;
CHAR_INFO g_BackBuffer[4000];
COORD g_WindowSize = {80,50};
COORD g_WindowPos = {0,0};
SMALL_RECT g_WindowRect = {0,0,79,49};
bool g_Running = true;
int g_FrameCounter = 0;

//////////////////////////////////////////////////////////////////////////
//  FRAMEWORK
//////////////////////////////////////////////////////////////////////////

void SetChar(int x, int y, char c, WORD a = 15)
{
	g_BackBuffer[(y * 80) + x].Char.AsciiChar = c;
	g_BackBuffer[(y * 80) + x].Attributes = a;
}

void Print(int x, int y, const char* c, int a = 15)
{
	int _Length = strlen(c);

	if(_Length <= 0)
		return;

	for(int i = 0; i < _Length; ++i)
	{
		SetChar(x + i, y, c[i], a);
	}
}

void SetPos(int x,int y)
{
	COORD c1={x,y};
	SetConsoleCursorPosition(g_WriteHandle, c1);
}

//////////////////////////////////////////////////////////////////////////
//  GAME-SPECIFIC
//////////////////////////////////////////////////////////////////////////

float GetDistance(float x1, float y1, float x2, float y2)
{
	return ( sqrt( pow( abs(x1 - x2), 2 ) + pow( abs(y1 - y2) , 2) ) );
}

struct Player
{
	int x;
	int y;
	int facing;
	int weapontype;
	int ammo;
	DWORD timeoflastmove;
	int speed;
	int health;
	int xp;
	int xptonextlevel;
	DWORD timeoflastattack;
	int lengthofattack;
	int attackcooldown;
	int wasdamagedlastturn;
};

Player g_Player;

struct Enemy
{
	int x;
	int y;
	bool aggrod;
	int speed;
	DWORD timeoflastmove;
	char sprite;
	DWORD timeoflastattack;
	int attackspeed;
	int wasdamagedlastturn;
	int health;
};

vector<Enemy> g_Enemies;

struct PowerUp
{
	int type;
	int x;
	int y;
	char sprite;
	int deltax;
	int deltay;
	int isdead;
};

vector<PowerUp> g_PowerUps;

struct Treasure
{
	int type;
	string name;
	int x;
	int y;
	int isfound;
	char sprite;
	string description;
};

vector<Treasure> g_Treasures;

int g_CurrentlySelectedTreasure;


BYTE g_Castle[200][200];

vector<string> g_Console;

vector<string> g_Intro;

vector<string> g_GameOver;

vector<string> g_WonGame;

vector<string> g_RunAways;

enum States
{
	STATE_OPENING = 0,
	STATE_PLAYING,
	STATE_GAMEOVER,
	STATE_WONGAME
};

int g_State = STATE_OPENING;

void InitLevel()
{
 	FILE* file;
 
 	file = fopen("castle.bmp", "rb");
 
 	//  Kill the header.
 	for(int i = 0; i < 1078; ++i)
 	{
 		BYTE okay = fgetc(file);
 	}
 
 	for(int i = 199; i > -1; --i)
 	{
 		for(int j = 0; j < 200; ++j)
 		{
 			g_Castle[j][i] = fgetc(file);
 		}
 	}

	fclose(file);

	ofstream outstream;
	outstream.open("castle.txt");

	for(int i = 0; i < 200; ++i)
	{
		for(int j = 0; j < 200; ++j)
		{
			if(g_Castle[j][i] == 0)
				g_Castle[j][i] = 1;
			else if(g_Castle[j][i] == 2)
				g_Castle[j][i] = 0;
			if(g_Castle[j][i] < 10)
				outstream << " ";
			outstream << int(g_Castle[j][i]);
			outstream << " ";
		}
		outstream << endl;
	}

	outstream.close();

	
	int numberofenemies = 25 + (rand() % 10);
	for(int i = 0; i < numberofenemies; ++i)
	{
		Enemy enemy;
		enemy.x = 10 + (rand() % 180);
		enemy.y = 10 + (rand() % 180);
		enemy.aggrod = false;
		enemy.speed = 100;
		enemy.attackspeed = 1000;
		enemy.timeoflastmove = 0;
		enemy.timeoflastattack = 0;
		enemy.wasdamagedlastturn = 0;
		enemy.health = 2;
		g_Enemies.push_back(enemy);
	}

	g_Player.x = 100;
	g_Player.y = 165;
	g_Player.facing = 1;
	g_Player.weapontype = 0;
	g_Player.ammo = 0;
	g_Player.timeoflastmove = 0;
	g_Player.speed = 50;
	g_Player.health = 3;
	g_Player.timeoflastattack = 0;
	g_Player.attackcooldown = 300;
	g_Player.lengthofattack = 100;
	g_Player.wasdamagedlastturn = 0;

	Treasure sword;
	sword.x = 100;
	sword.y = 155;
	sword.name = "Longsword";
	sword.sprite = '!';
	sword.isfound = false;
	sword.description = "Despite its age, there is no rust - and it still looks quite sharp.";
	sword.type = 0;
	g_Treasures.push_back(sword);

	Treasure bow;
	bow.x = 30;
	bow.y = 30;
	bow.name = "Bow";
	bow.sprite = 'D';
	bow.isfound = false;
	bow.description = "A stout wooden bow, with a quiver full of arrows.";
	bow.type = 1;
	g_Treasures.push_back(bow);

	Treasure treasure;
	treasure.x = 100;
	treasure.y = 100;
	treasure.name = "Harp";
	treasure.sprite = 14;
	treasure.isfound = false;
	treasure.description = "A beautiful, delicate glass harp.";
	treasure.type = 2;
	g_Treasures.push_back(treasure);

	treasure.x = 153;
	treasure.y = 29;
	treasure.name = "Ring";
	treasure.sprite = 9;
	treasure.isfound = false;
	treasure.description = "Through a grate in the floor you can see a ring...but you can't get it.";
	treasure.type = 3;
	g_Treasures.push_back(treasure);

	treasure.x = 163;
	treasure.y = 71;
	treasure.name = "Jade Figurine";
	treasure.sprite = 157;
	treasure.isfound = false;
	treasure.description = "A strange figurine carved from jade - locked inside a glass case.";
	treasure.type = 4;
	g_Treasures.push_back(treasure);

	treasure.x = 34;
	treasure.y = 73;
	treasure.name = "Serpent Statue";
	treasure.sprite = 21;
	treasure.isfound = false;
	treasure.description = "A silver statue of intertwined serpents.";
	treasure.type = 5;
	g_Treasures.push_back(treasure);

	treasure.x = 157;
	treasure.y = 169;
	treasure.name = "Gold Bars";
	treasure.sprite = 240;
	treasure.isfound = false;
	treasure.description = "They're bars.  Made of gold.  Must be worth a fortune.";
	treasure.type = 6;
	g_Treasures.push_back(treasure);

	treasure.x = 31;
	treasure.y = 45;
	treasure.name = "Strange Rock";
	treasure.sprite = 7;
	treasure.isfound = false;
	treasure.description = "It's a smooth rock.  It seems to want to pull out of your hand.";
	treasure.type = 7;
	g_Treasures.push_back(treasure);

	g_CurrentlySelectedTreasure = -1;
}

void AddStringToConsole(string stringtoadd)
{
	g_Console.push_back(stringtoadd);
	if(g_Console.size() > 9)
	{
		g_Console.erase(g_Console.begin());
	}
}

void DrawBackground()
{
	for(int i = 0; i < 80; ++i)
	{
		for(int j = 0; j < 50; ++j)
		{
			if(j == 0 || j == 39 || j == 49)
			{
				SetChar(i, j, 219);
			}
			else
			{
				SetChar(0, j, 219);
				SetChar(79, j, 219);
				if(j < 40)
					SetChar(59, j, 219);
			}
		}
	}

	Print(32, 0, "CASTLE ADVENTURE", 15);
}

void DrawMapData()
{
	for(int i = g_Player.x - 29; i < g_Player.x + 29; ++i)
	{
		for(int j = g_Player.y - 19; j < g_Player.y + 19; ++j)
		{
			if(i >= 0 && i <= 199 && j >= 0 && j <= 199)
			{
				switch(g_Castle[i][j])
				{
				case 1:
					{
						if(i - (g_Player.x - 29) >= 0 && i - (g_Player.x - 29) < 59 &&
							j - (g_Player.y - 19) >= 0 && j - (g_Player.y - 19) < 39)
						{
							SetChar(i - (g_Player.x - 30), j - (g_Player.y - 20), 178);
						}
					}
					break;
				}
			}
		}
	}

}

void DrawEntities()
{
	//  Draw the player
	if(g_Player.wasdamagedlastturn > 0)
	{
		SetChar(30, 20, 1, 12);
		--g_Player.wasdamagedlastturn;
	}
	else
	{
		SetChar(30, 20, 1, 15);
	}

	//  Draw all monsters
	for(UINT i = 0; i < g_Enemies.size(); ++i)
	{

		if(g_Enemies[i].x - (g_Player.x - 29) >= 0 && g_Enemies[i].x - (g_Player.x - 29) < 59 &&
			g_Enemies[i].y - (g_Player.y - 19) >= 0 && g_Enemies[i].y - (g_Player.y - 19) < 39)
		{
			if(g_Enemies[i].wasdamagedlastturn > 0)
			{
				SetChar(g_Enemies[i].x - (g_Player.x - 30), g_Enemies[i].y - (g_Player.y - 20), 12, 12);
				--g_Enemies[i].wasdamagedlastturn;
			}
			else
			{
				SetChar(g_Enemies[i].x - (g_Player.x - 30), g_Enemies[i].y - (g_Player.y - 20), 12, 15);
			}
		}
	}

	//  Draw all the treasures
	for(UINT i = 0; i < g_Treasures.size(); ++i)
	{
		if(g_Treasures[i].isfound == false)
		{	
			if(g_Treasures[i].x - (g_Player.x - 29) >= 0 && g_Treasures[i].x - (g_Player.x - 29) < 59 &&
				g_Treasures[i].y - (g_Player.y - 19) >= 0 && g_Treasures[i].y - (g_Player.y - 19) < 39)
			{
				SetChar(g_Treasures[i].x - (g_Player.x - 30), g_Treasures[i].y - (g_Player.y - 20), g_Treasures[i].sprite);
			}
		}
	}

	//  Draw Powerups
	for(vector<PowerUp>::iterator node = g_PowerUps.begin(); 
		node != g_PowerUps.end(); ++node)
	{
		if((*node).x - (g_Player.x - 29) >= 0 && (*node).x - (g_Player.x - 29) < 59 &&
			(*node).y - (g_Player.y - 19) >= 0 && (*node).y - (g_Player.y - 19) < 39)
		{
			SetChar((*node).x - (g_Player.x - 30), (*node).y - (g_Player.y - 20), (*node).sprite);
		}
	}
}

void DrawStats()
{
	Print(67, 1, "HEALTH", 15);
	for(int i = 0; i < g_Player.health; ++i)
	{
		SetChar(60 + i, 2, 3);
	}

	Print(65, 5, "TREASURES", 15);

	int counter = 0;
	for(int i = 0; i < g_Treasures.size(); ++i)
	{
		if( g_Treasures[i].isfound )
		{
			SetChar(63, 6 + counter, g_Treasures[i].sprite, 15 );
			Print(65, 6 + counter, g_Treasures[i].name.c_str(), 15 );

			if(i == g_CurrentlySelectedTreasure)
			{
				SetChar(61, 6 + counter,'>');
			}
			++counter;
		}
	}
}

void DrawConsole()
{
	int counter = 1;
	for(vector<string>::iterator node = g_Console.begin(); node != g_Console.end(); ++node)
	{
		Print(1, 39 + counter, (*node).c_str(), 15);
		++counter;
	}

}

bool g_IsUpDown;
bool g_IsLeftDown;
bool g_IsRightDown;
bool g_IsDownDown;
bool g_ZKeypress;
bool g_AKeypress;
bool g_SKeypress;
bool g_AnyKeypress;

void Init()
{
	g_WriteHandle = GetStdHandle(STD_OUTPUT_HANDLE);
	g_ReadHandle = GetStdHandle(STD_INPUT_HANDLE);
	SetConsoleWindowInfo(g_WriteHandle, TRUE, &g_WindowRect);
	SetConsoleScreenBufferSize(g_WriteHandle, g_WindowSize);

	g_Enemies.clear();
	g_Treasures.clear();
	g_Console.clear();
	g_PowerUps.clear();

	InitLevel();

	g_Console.push_back("Welcome to Castle Adventure!");

	g_IsUpDown = false;
	g_IsLeftDown = false;
	g_IsRightDown = false;
	g_IsDownDown = false;
	g_ZKeypress = false;
	g_AKeypress = false;
	g_SKeypress = false;
	g_AnyKeypress = false;

	g_Intro.clear();
	g_Intro.push_back("Welcome to Castle Adventure!");
	g_Intro.push_back("Castle Adventure is a simple (VERY simple)");
	g_Intro.push_back("action-RPG game.");
	g_Intro.push_back(" ");
	g_Intro.push_back("Your goal is to collect all the treasures in the castle.");
	g_Intro.push_back(" ");
	g_Intro.push_back("Some treasures are weapons, and will allow you to");
	g_Intro.push_back("fight the monsters in the castle.");
	g_Intro.push_back(" ");
	g_Intro.push_back("Other treasures will give you benefits, like making your");
	g_Intro.push_back("weapons stronger or allowing you to take more damage.");
	g_Intro.push_back(" ");
	g_Intro.push_back("And some treasures must be used to get other treasures.");
	g_Intro.push_back(" ");
	g_Intro.push_back("You move around with the arrow keys, and you use");
	g_Intro.push_back("whatever treasure you have selected when you press the");
	g_Intro.push_back("'Z' key.  Pressing 'A' and 'S' allows you to cycle");
	g_Intro.push_back("through the treasures you've collected.");
	g_Intro.push_back(" ");
	g_Intro.push_back("Press any key to begin!");
	g_Intro.push_back(" ");
	g_Intro.push_back("(Uh...except Escape.  That always exits the game.)");


	g_GameOver.clear();
	g_GameOver.push_back("You have come to an unfortunate end.");
	g_GameOver.push_back(" ");
	g_GameOver.push_back("How unfortunate.");
	g_GameOver.push_back(" ");
	g_GameOver.push_back("Press any key to start a new game!");
	g_GameOver.push_back(" ");
	g_GameOver.push_back("(Um...except Escape, which still quits.)");

	g_WonGame.clear();
	g_WonGame.push_back("Wonderful!  You have collected all treasures!");
	g_WonGame.push_back(" ");
	g_WonGame.push_back("YOU HAVE WON THE GAME!");
	g_WonGame.push_back(" ");
	g_WonGame.push_back("Press any key to start a new game!");
	g_WonGame.push_back(" ");
	g_WonGame.push_back("(Um...except Escape, which...well, you know.)");

}



void Input()
{
	DWORD nE = 0;
	DWORD nER = 0;

	g_ZKeypress = false;
	g_AKeypress = false;
	g_SKeypress = false;
	g_AnyKeypress = false;

	GetNumberOfConsoleInputEvents(g_ReadHandle, &nE);
	if(nE != 0)
	{
		INPUT_RECORD *eB = new INPUT_RECORD[nE];
		ReadConsoleInput(g_ReadHandle, eB, nE, &nER);
		for(DWORD i = 0; i < nER; ++i)
		{
			if (eB[i].EventType == KEY_EVENT)
			{
				if(eB[i].Event.KeyEvent.wVirtualKeyCode == VK_ESCAPE)
				{
					g_Running = false;
				}
			}

			if (eB[i].EventType == KEY_EVENT && eB[i].Event.KeyEvent.bKeyDown)
			{
				if(eB[i].Event.KeyEvent.wVirtualKeyCode == VK_LEFT)
				{
					g_IsLeftDown = true;
				}

				if(eB[i].Event.KeyEvent.wVirtualKeyCode == VK_RIGHT)
				{
					g_IsRightDown = true;
				}

				if(eB[i].Event.KeyEvent.wVirtualKeyCode == VK_UP)
				{
					g_IsUpDown = true;
				}

				if(eB[i].Event.KeyEvent.wVirtualKeyCode == VK_DOWN)
				{
					g_IsDownDown = true;
				}
			}
			else if (eB[i].EventType == KEY_EVENT && !eB[i].Event.KeyEvent.bKeyDown)
			{
				if(eB[i].Event.KeyEvent.wVirtualKeyCode == VK_LEFT)
				{
					g_IsLeftDown = false;
				}

				if(eB[i].Event.KeyEvent.wVirtualKeyCode == VK_RIGHT)
				{
					g_IsRightDown = false;
				}

				if(eB[i].Event.KeyEvent.wVirtualKeyCode == VK_UP)
				{
					g_IsUpDown = false;
				}

				if(eB[i].Event.KeyEvent.wVirtualKeyCode == VK_DOWN)
				{
					g_IsDownDown = false;
				}

				if(eB[i].Event.KeyEvent.wVirtualKeyCode == 'Z' || eB[i].Event.KeyEvent.wVirtualKeyCode == 'z')
				{
					g_ZKeypress = true;
				}

				if(eB[i].Event.KeyEvent.wVirtualKeyCode == 'A' || eB[i].Event.KeyEvent.wVirtualKeyCode == 'a')
				{
					g_AKeypress = true;
				}

				if(eB[i].Event.KeyEvent.wVirtualKeyCode == 'S' || eB[i].Event.KeyEvent.wVirtualKeyCode == 's')
				{
					g_SKeypress = true;
				}

				g_AnyKeypress = true;
			}

			else if(eB[i].EventType == MOUSE_EVENT)
			{
				eB[i].Event.MouseEvent.dwMousePosition.X;
				eB[i].Event.MouseEvent.dwMousePosition.Y;
				if(eB[i].Event.MouseEvent.dwButtonState
					& FROM_LEFT_1ST_BUTTON_PRESSED)
				{

				}
			}
		}
		delete eB;
	}
}


void UpdatePlayer()
{
	//  First, check for game over.
	bool didwingame = true;
	for(int i = 0; i < g_Treasures.size(); ++i)
	{
		didwingame &= g_Treasures[i].isfound;
	}
	if(didwingame)
	{
		g_State = STATE_WONGAME;
		return;
	}


	for(int i = 0; i < g_Treasures.size(); ++i)
	{
		if(g_Player.x == g_Treasures[i].x && g_Player.y == g_Treasures[i].y && !g_Treasures[i].isfound && g_Treasures[i].type != 3 && g_Treasures[i].type != 4)
		{
			string update = "You found the ";
			update += g_Treasures[i].name;
			update += "!";
			AddStringToConsole(update);
			AddStringToConsole( g_Treasures[i].description );
			//  Mark this treasure taken.
			g_Treasures[i].isfound = true;

			// If this is the first treasure we've found, select it.
			if(g_CurrentlySelectedTreasure == -1)
			{
				g_CurrentlySelectedTreasure = i;
				string update = "You equip the ";
				update += g_Treasures[i].name;
				update += ".";
				AddStringToConsole(update);
			}
		}
		else if(g_Player.x == g_Treasures[i].x && g_Player.y == g_Treasures[i].y && g_Treasures[i].type == 3 && !g_Treasures[i].isfound)
		{
			AddStringToConsole( g_Treasures[i].description );
		}
		else if(g_Player.x == g_Treasures[i].x && g_Player.y == g_Treasures[i].y && g_Treasures[i].type == 4 && !g_Treasures[i].isfound)
		{
			AddStringToConsole( g_Treasures[i].description );
		}
	}

	for(vector<PowerUp>::iterator node = g_PowerUps.begin();
		node != g_PowerUps.end(); )
	{
		if(g_Player.x == (*node).x && g_Player.y == (*node).y)
		{
			switch((*node).type)
			{
			case 0:
				string update = "You picked up a heart!";
				if(g_Player.health < 17)
				{
					AddStringToConsole("You gain a point of health!");
					++g_Player.health;
				}
				else
				{
					AddStringToConsole("But you are already at maximum health.");
				}
				break;
			}
			node = g_PowerUps.erase(node);
		}
		else
		{
			++node;
		}
	}

	int targetx = g_Player.x;
	int targety = g_Player.y;

	if(g_IsLeftDown)
	{
		if(g_Player.x > 0)
		{
			targetx = g_Player.x - 1;
		}
		g_Player.facing = 0;
	}

	if(g_IsRightDown)
	{
		if(g_Player.x < 199)
		{
			targetx = g_Player.x + 1;
		}
		g_Player.facing = 1;
	}

	if(g_IsUpDown)
	{
		if(g_Player.y > 0)
		{
			targety = g_Player.y - 1;
		}
		g_Player.facing = 2;
	}

	if(g_IsDownDown)
	{
		if(g_Player.y < 199)
		{
			targety = g_Player.y + 1;
		}
		g_Player.facing = 3;
	}

	
	//  Make sure we're not walking into a wall, a treasure, the player,
	//  or another monster.

	bool goodtarget = true;

	if(g_Castle[targetx][targety] != 0)
		goodtarget = false;

	for(vector<Enemy>::iterator node = g_Enemies.begin(); node != g_Enemies.end(); ++node)
	{
		if(targetx == (*node).x && targety == (*node).y)
		{
			goodtarget = false;
		}
	}

	if(goodtarget && (timeGetTime() - g_Player.timeoflastmove > g_Player.speed)	)
	{
		g_Player.x = targetx;
		g_Player.y = targety;
		g_Player.timeoflastmove = timeGetTime();
	}

	//  Walking's handled.  Now it's time to handle using treasures
	if(g_AKeypress)
	{
		if(g_CurrentlySelectedTreasure != -1)
		{
			//  Find the next "found" treasure
			for(int i = g_Treasures.size() - 1; i >= 0; --i)
			{
				if(g_Treasures[i].isfound && i < g_CurrentlySelectedTreasure)
				{
					g_CurrentlySelectedTreasure = i;
					break;
				}
			}
		}
	}

	if(g_SKeypress)
	{
		if(g_CurrentlySelectedTreasure != -1)
		{
			//  Find the next "found" treasure
			for(int i = 0; i < g_Treasures.size(); ++i)
			{
				if(g_Treasures[i].isfound && i > g_CurrentlySelectedTreasure)
				{
					g_CurrentlySelectedTreasure = i;
					break;
				}
			}
		}
	}

	if(g_ZKeypress)
	{
		if(g_CurrentlySelectedTreasure == -1)
		{
			AddStringToConsole("You have no treasure equipped.");
		}

		else
		{
			switch(g_Treasures[g_CurrentlySelectedTreasure].type)
			{
			case 0: //  Longsword
				{
					if(timeGetTime() - g_Player.timeoflastattack > g_Player.attackcooldown)
					{
						g_Player.timeoflastattack = timeGetTime();
					}
					break;
				}

			case 1: //  Bow
				{
					if(g_Player.facing == 0)
					{
						PowerUp powerup;
						powerup.x = g_Player.x - 1;
						powerup.y = g_Player.y;
						powerup.type = 1;
						powerup.sprite = 15;
						powerup.deltax = -1;
						powerup.deltay = 0;
						powerup.isdead = false;
						g_PowerUps.push_back(powerup);
					}
					if(g_Player.facing == 1)
					{
						PowerUp powerup;
						powerup.x = g_Player.x + 1;
						powerup.y = g_Player.y;
						powerup.type = 1;
						powerup.sprite = 15;
						powerup.deltax = +1;
						powerup.deltay = 0;
						powerup.isdead = false;
						g_PowerUps.push_back(powerup);
					}
					if(g_Player.facing == 2)
					{
						PowerUp powerup;
						powerup.x = g_Player.x;
						powerup.y = g_Player.y - 1;
						powerup.type = 1;
						powerup.sprite = 15;
						powerup.deltax = 0;
						powerup.deltay = -1;
						powerup.isdead = false;
						g_PowerUps.push_back(powerup);
					}
					if(g_Player.facing == 3)
					{
						PowerUp powerup;
						powerup.x = g_Player.x;
						powerup.y = g_Player.y + 1;
						powerup.type = 1;
						powerup.sprite = 15;
						powerup.deltax = 0;
						powerup.deltay = +1;
						powerup.isdead = false;
						g_PowerUps.push_back(powerup);
					}


					break;
				}

			case 2:  //  Harp
				
				{
					AddStringToConsole( "You play a glissando on the glass harp." );
					if(g_Player.x == 163 && g_Player.y == 71)
					{
						AddStringToConsole( "It shatters the glass case!" );
						AddStringToConsole( "You have found a jade figurine!");
						g_Treasures[4].isfound = true;
					}
				}
				break;

			case 7:  //  Stone
				{
					if(g_Player.x == 153 && g_Player.y == 29)
					{
						AddStringToConsole( "The strange rock is a magnet!" );
						AddStringToConsole( "It pulls the ring up through the grate.");
						AddStringToConsole( "You have found a diamond ring!");
						g_Treasures[3].isfound = true;
					}

					AddStringToConsole( "The rock seems to want to pull out of your hand." );
				}
				break;

			default:
				AddStringToConsole( "That doesn't seem to do anything." );
				break;


			}
		}
	}

	//  Are we still attacking?
	if(timeGetTime() - g_Player.timeoflastattack < g_Player.lengthofattack)
	{
		switch(g_Player.facing)
		{
		case 0: //  Left
			SetChar(29, 20, '-');
			//  See if there's a monster in the list that is 1 or
			//  2 spaces up from the player
			for(int i = 0; i < g_Enemies.size(); ++i)
			{
				if(g_Enemies[i].x == g_Player.x - 1 && g_Enemies[i].y == g_Player.y)
				{
					--g_Enemies[i].health;
					g_Enemies[i].wasdamagedlastturn = 10;
				}
			}
			//  Hurt that monster
		break;

		case 1: //  RIGHT
			SetChar(31, 20, '-');
			//  See if there's a monster in the list that is 1 or
			//  2 spaces up from the player
			for(int i = 0; i < g_Enemies.size(); ++i)
			{
				if(g_Enemies[i].x == g_Player.x + 1 && g_Enemies[i].y == g_Player.y)
				{
					--g_Enemies[i].health;
					g_Enemies[i].wasdamagedlastturn = 10;
				}
			}
			//  Hurt that monster
		break;

		case 2: //  Up
			SetChar(30, 19, '|');
			//  See if there's a monster in the list that is 1 or
			//  2 spaces up from the player
			for(int i = 0; i < g_Enemies.size(); ++i)
			{
				if(g_Enemies[i].x == g_Player.x && g_Enemies[i].y == g_Player.y - 1)
				{
					--g_Enemies[i].health;
					g_Enemies[i].wasdamagedlastturn = 10;
				}
			}
			//  Hurt that monster
			break;

		case 3: //  Down
			SetChar(30, 21, '|');
			//  See if there's a monster in the list that is 1 or
			//  2 spaces up from the player
			for(int i = 0; i < g_Enemies.size(); ++i)
			{
				if(g_Enemies[i].x == g_Player.x && g_Enemies[i].y == g_Player.y + 1)
				{
					--g_Enemies[i].health;
					g_Enemies[i].wasdamagedlastturn = 10;
				}
			}
			//  Hurt that monster
		break;
		}
	}
}

void CheckTriggers()
{

}

void UpdateMonsters()
{
	for( vector<Enemy>::iterator node = g_Enemies.begin(); node != g_Enemies.end(); )
	{
		if( (*node).health == 0 )
		{
			if(rand()%3 != 0)
			{
				PowerUp heart;
				heart.x = (*node).x;
				heart.y = (*node).y;
				heart.type = 0;
				heart.sprite = 3;
				heart.deltax = 0;
				heart.deltay = 0;
				heart.isdead = false;
				g_PowerUps.push_back(heart);
			}
			node = g_Enemies.erase(node);
			AddStringToConsole("You have killed a monster!");

		}
		else
		{
			++node;
		}
	}

	for(vector<Enemy>::iterator node = g_Enemies.begin(); node != g_Enemies.end(); ++node)
	{
		//  If monsters are close enough to the player, they aggro.
		if(GetDistance( (*node).x, (*node).y, g_Player.x, g_Player.y) <= 1)
		{
			//  Hurt player
			if((timeGetTime() - (*node).timeoflastattack) > (*node).attackspeed)
			{
				--g_Player.health;
				g_Player.wasdamagedlastturn = true;

				AddStringToConsole( "The monster attacks you!" );
				(*node).timeoflastattack = timeGetTime();
			}
		}
		else if(GetDistance( (*node).x, (*node).y, g_Player.x, g_Player.y) <= 12 && (*node).aggrod == false )
		{
			
			AddStringToConsole("A monster has spotted you!");
			(*node).aggrod = true;
			(*node).timeoflastmove = timeGetTime();

		}
		else if(GetDistance( (*node).x, (*node).y, g_Player.x, g_Player.y) <= 12 && (*node).aggrod == true  )
		{
			if((timeGetTime() - (*node).timeoflastmove) > (*node).speed)
			{
				int targetx, targety;
				if(g_Player.x > (*node).x ) targetx = (*node).x + 1;
				else if(g_Player.x < (*node).x ) targetx = (*node).x - 1;
				else targetx = g_Player.x;
				if(g_Player.y > (*node).y ) targety = (*node).y + 1;
				else if(g_Player.y < (*node).y ) targety = (*node).y - 1;
				else targety = g_Player.y;


				//  Make sure we're not walking into a wall, a treasure, the player,
				//  or another monster.

				bool goodtarget = true;

				if(g_Castle[targetx][targety] != 0)
					goodtarget = false;

				if( (*node).x == g_Player.x && (*node).y == g_Player.y)
				{
					goodtarget = false;
				}

				for(vector<Enemy>::iterator node = g_Enemies.begin(); node != g_Enemies.end(); ++node)
				{
					if(targetx == (*node).x && targety == (*node).y)
					{
						goodtarget = false;
					}
				}

				for(vector<Treasure>::iterator node = g_Treasures.begin(); node != g_Treasures.end(); ++node)
				{
					if(targetx == (*node).x && targety == (*node).y)
					{
						goodtarget = false;
					}
				}

				if(goodtarget)
				{
					(*node).x = targetx;
					(*node).y = targety;
					(*node).timeoflastmove = timeGetTime();
				}
			}
		}
		else if( (*node).aggrod == true )
		{
			//  Player's too far away, we lose aggro.
			(*node).aggrod = false;
			switch(rand()%6)
			{
			case 0:
				AddStringToConsole( "The monster refuses to run after you any more.  It has its dignity." );
				break;
			case 1:
				AddStringToConsole( "Panting and sweating, the monster stops chasing you." );
				break;
			case 2:
				AddStringToConsole( "The monster is contractually obligated not to chase you any more.");
				break;
			case 3:
				AddStringToConsole( "The monster decides that it really didn't want to eat you anyway.");
				break;
			case 4:
				AddStringToConsole( "The monster thinks it's rather unfair that you are so much faster than it.");
				break;
			case 5:
				AddStringToConsole( "The monster curses its stubby little legs. ");
				break;
			}
			
		}
	}
}

void UpdatePowerUps()
{
	for( vector<PowerUp>::iterator node = g_PowerUps.begin(); node != g_PowerUps.end(); ++node )
	{
		//  Move them if they are moving.
		(*node).x += (*node).deltax;
		(*node).y += (*node).deltay;

		if(g_Castle[(*node).x][(*node).y] != 0)
		{
			(*node).isdead = true;
		}
		else
		{
			//  If it's a weapon, check for monster hits.
			if( (*node).type == 1 )
			{
				for(int i = 0; i < g_Enemies.size(); ++i)
				{
					if(g_Enemies[i].x == (*node).x && g_Enemies[i].y == (*node).y)
					{
						--g_Enemies[i].health;
						g_Enemies[i].wasdamagedlastturn = 10;
						(*node).isdead = true;
					}
				}
			}
		}
	}

	for( vector<PowerUp>::iterator node = g_PowerUps.begin(); node != g_PowerUps.end(); )
	{
		if( (*node).isdead )
		{
			node = g_PowerUps.erase(node);
		}
		else
		{
			++node;
		}
	}
}



void Update()
{
	++g_FrameCounter;

	UpdatePlayer();

	CheckTriggers();

	UpdateMonsters();

	UpdatePowerUps();

	if(g_Player.health <= 0)
	{
		g_State = STATE_GAMEOVER;
	}
}

void UpdateOpening()
{
	if(g_AnyKeypress)
	{
		g_State = STATE_PLAYING;
	}
};

void DrawOpening()
{
	DrawBackground();

	DrawStats();

	for(int i = 0; i < g_Intro.size(); ++i)
	{
		Print(30 - (g_Intro[i].length() / 2), 5 + i, g_Intro[i].c_str());
	}

	WriteConsoleOutput(g_WriteHandle, g_BackBuffer, g_WindowSize, g_WindowPos, &g_WindowRect);

}

void UpdateGameOver()
{

	if(g_AnyKeypress)
	{
		Init();
		g_State = STATE_PLAYING;
	}
};

void DrawGameOver()
{

	DrawBackground();

	DrawStats();

	DrawConsole();

	for(int i = 0; i < g_GameOver.size(); ++i)
	{
		Print(30 - (g_GameOver[i].length() / 2), 10 + i, g_GameOver[i].c_str());
	}


	WriteConsoleOutput(g_WriteHandle, g_BackBuffer, g_WindowSize, g_WindowPos, &g_WindowRect);
	static bool first = true;
	if(first)
	{
		Sleep(1000);
		first = false;
	}
}

void DrawWonGame()
{

	DrawBackground();

	DrawStats();

	DrawConsole();

	for(int i = 0; i < g_WonGame.size(); ++i)
	{
		Print(30 - (g_WonGame[i].length() / 2), 10 + i, g_WonGame[i].c_str());
	}


	WriteConsoleOutput(g_WriteHandle, g_BackBuffer, g_WindowSize, g_WindowPos, &g_WindowRect);
	static bool first = true;
	if(first)
	{
		Sleep(1000);
		first = false;
	}
}

void Draw()
{

	DrawBackground();

	DrawMapData();

	DrawEntities();

	DrawStats();

	DrawConsole();





	WriteConsoleOutput(g_WriteHandle, g_BackBuffer, g_WindowSize, g_WindowPos, &g_WindowRect);
}




int main(int argc,char* argv[])
{
	Init();



	while(g_Running)
	{
		ZeroMemory(g_BackBuffer,4000*sizeof(CHAR_INFO));

		switch(g_State)
		{
		case STATE_OPENING:
			{
				Input();

				UpdateOpening();

				DrawOpening();
			}
			break;

		case STATE_PLAYING:
			{
				Input();

				Update();

				Draw();
			}
			break;

		case STATE_GAMEOVER:
			{
				Input();

				UpdateGameOver();

				DrawGameOver();
			}
			break;

		case STATE_WONGAME:
			{
				Input();

				UpdateGameOver();

				DrawWonGame();
			}
			break;
		}

		Sleep(17);
	}

	return 0;
}
